import React, { ChangeEvent } from "react";
import { saveGraphJson } from "../../common/datasets";
import "./historynavigator.css";
import { History } from "../../common/history";

type propTypes = {
    history: History<any>;
    spaceId: string;
    onChange: { (): void };
};

export class HistoryNavigator extends React.Component<propTypes> {
    constructor(props: propTypes) {
        super(props);

        this.handleUndo = this.handleUndo.bind(this);
        this.handleRedo = this.handleRedo.bind(this);
        this.handleSave = this.handleSave.bind(this);
        this.handleSelectSavepoint = this.handleSelectSavepoint.bind(this);
    }

    /**
     * Undo a step in the history object.
     */
    handleUndo() {
        this.props.history.undo();
    }

    /**
     * Redo a step in the hisory object.
     */
    handleRedo() {
        this.props.history.redo();
    }

    /**
     * Saves current data of history object.
     */
    handleSave() {
        saveGraphJson(
            this.props.spaceId,
            this.props.history.currentCheckpoint.data
        );
        // TODO: Save successful?
        // this.props.historyObject.markChangesAsSaved(); TODO: Reimplement
        alert(
            "Saved! (Though not for sure, currently not checking for success of failure)"
        );
    }

    /**
     * Loads selected savepoint into history object.
     */
    handleSelectSavepoint(e: ChangeEvent<HTMLSelectElement>) {
        if (!this.props.history.resetToCheckpoint(Number(e.target.value))) {
            alert("Something went wrong, could not load savepoint.");
        } else {
            this.props.onChange();
        }
    }

    render(): React.ReactNode {
        return (
            <div id="history-navigator">
                <button onClick={this.handleUndo}>Undo</button>
                <button onClick={this.handleRedo}>Redo</button>
                <select
                    onChange={this.handleSelectSavepoint}
                    value={
                        this.props.history
                            ? this.props.history.currentCheckpoint.id
                            : 0
                    }
                >
                    {this.props.history
                        ? this.props.history.checkpoints.map((savepoint) => {
                              return (
                                  <option
                                      key={savepoint.id}
                                      value={savepoint.id}
                                  >
                                      {savepoint.description}
                                  </option>
                              );
                          })
                        : ""}
                </select>
                <button
                    onClick={this.handleSave}
                    // disabled={ TODO: Reimplement
                    //     this.props.history
                    //         ? !this.props.history.hasUnsavedChanges()
                    //         : true
                    // }
                >
                    Save
                </button>
            </div>
        );
    }
}
